/*
  mergeproxyuniverse.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "universe.h"

#include <memory>
#include <unordered_map>

namespace SACN {

/**
 * @brief The MergeProxyUniverse class
 *  6.2.3.1 Multiple Sources at Highest Priority
 *    It is possible for there to be multiple sources, all transmitting data
 *    at the highest currently active priority for a given universe. When this
 *    occurs, receivers must handle these sources in some way.
 */
class MergeProxyUniverse
  : public SACN::Universe
{
public:
    MergeProxyUniverse();
    ~MergeProxyUniverse();

    // Source universes:
    const std::vector<Provenance> sources() const;
    Universe* sourceUniverse(const Provenance&);
    bool hasSourceUniverse(const Provenance&) const;
    bool hasSources() const;
    void deleteSourceUniverse(const Provenance&);
    void addSourceUniverse(Universe *);
    void dataChangedNotifier(DMX::Universe* universe);

    // SACN::Universe overrides:
    bool isSyncronized() const override;
    std::shared_ptr<Provenance> source() const override;
    void set(std::shared_ptr<ACN::DMP::Pdu>, std::shared_ptr<Provenance>) override;
    void setProvenance(std::shared_ptr<Provenance>) override {};
    void synchronize() override;

    // DMX::Universe Overrides:
    const DMX::DimmerData *data() const override;
    uint8_t slot(const uint16_t) override;
    double rxRate() override;

private:
    std::unordered_map<Provenance, SACN::Universe*> sources_;

    Universe* dominant_() const;
    void newProvenance_(const Provenance&);
};

} // SACN namespace
